﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.IO.Ports;
using System.Drawing; //【Point】
using System.Drawing.Imaging;

namespace NTL.KinectLib
{
    public class ADconverter
    {
        //【シリアル通信関連】
        public const int nCH = 6;            //【A/Dコンバータの総チャンネル数】
        public const int nBytes = 2 * nCH;   //【受信データブロックバイト数】
        public const int nSPS = 800;         //【サンプリング速度】
        public const byte CMD_Start = 0x0C0; //【サンプリング開始コマンド】 bit7:TxD_Start, bit6: 0:16bit(50000digit),100SPS, 1:13bit(1250digit),800SPS 
        public const byte CMD_Stop = 0x00;   //【サンプリング終了コマンド】
        public const int TxBufSize = 256;    // 計測開始/終了コマンドだけ
        public const int RxBufSize = 65536;  // 6ch*2bytes*800SPS＝9600
        public const int nData = 160;             //【0.2秒分のサンプル数】
        public const int nBlock = nBytes * nData; //【0.2秒分の受信バイト数】
        public string[] PortName;                    //【ポートリスト】
        public string COMPORT;                       //【A/Dコンバータ通信用ComPort番号】
        public SerialPort Sensor = new SerialPort(); //【ADC用シリアルポート】
        public byte[] TxBuf = new byte[TxBufSize];   //【送信バッファ】
        public byte[] RxBuf = new byte[RxBufSize];   //【受信バッファ】
        public MovingAverage[] CH = new MovingAverage[nCH];
        public DisplayWave[] Disp = new DisplayWave[4];
        public EstimateParameter mEst = new EstimateParameter();
        public NoiseReduction Est = new NoiseReduction();
        public QRSFilter HBR = new QRSFilter();
        public int ReceivedBytes;

        public Boolean[] fRev = { true, false, false, false, false, false }; //【符号反転フラグ】ECGだけ符号反転
        public const int Range = 6250;
        public Boolean[] Chk = { true, false, true, true }; //【波形表示チェック】
        public int[] Zoom = { 2, 2, 2, 0 };                 //【波形表示倍率】
        public int nSample;
        public int nSampleCount;
        public Boolean Flag;
        public const string title = "No., Time[秒], ECG, ECG', QRS, HBR, 赤外線脈波\r\n";
        public double[] Sample0; //【ECG生波形】
        public double[] Sample1; //【基底遷移ノイズ除去】
        public double[] Sample2; //【矩形波相関フィルタ】
        public double[] Sample3; //【心拍数】
        public double[] Sample4; //【赤外線脈波】

        public Boolean Initialize()
        {
            Flag = false;
            ClearSampleData(800 * 120); //【800SPS x 100秒】

            for (int i = 0; i < nCH; i++) { CH[i] = new MovingAverage(); CH[i].Initialize(); }
            for (int i = 0; i < Disp.Length; i++) { Disp[i] = new DisplayWave(); Disp[i].Initialize(); }
            Est.Initialize(40);
            mEst.Initialize(40, nSPS);
            HBR.Initialize(); //【符号反転フラグ：反転】
            
            PortName = SerialPort.GetPortNames();       //【利用可能なシリアルポートのリストを取得】
            if (PortName.Length == 0) { return false; }

            int j = -1;
            Sensor.ReceivedBytesThreshold = nBytes; //【最低12バイト必要】16bit/ch×6ch=12Bytes
            for (int i = 0; i < PortName.Length; i++)
            {
                if (InitializeSerialPort(i)) 
                {
                    System.Threading.Thread.Sleep(500); //【0.100秒Sleep】
                    SendCommand(CMD_Start);
                    System.Threading.Thread.Sleep(100);  //【0.050秒Sleep】
                    if (Sensor.BytesToRead > 0)
                    {
                        TerminateSerialPort();
                        j = i; 
                        break;
                    }
                    else
                    {
                        Sensor.Close();
                    }
                }
            }
            if (j == -1) { return false; }

            Sensor.ReceivedBytesThreshold = nBlock;     //【最低12バイト必要】16bit/ch×6ch
            InitializeSerialPort(j);                    //
            Sensor.DataReceived += Sensor_DataReceived; //【データ受診イベントの設定】
            System.Threading.Thread.Sleep(100);         //【0.10秒Sleep】
            SendCommand(CMD_Start);
            return true;
        }

        //***********************************************************************
        //【利用可能なシリアルポートの探索処理】
        //***********************************************************************
        public Boolean SearchSerialPort()
        {
            PortName = SerialPort.GetPortNames();       //【利用可能なシリアルポートのリストを取得】
            if (PortName.Length == 0) { return false; }
            return true;
        }

        //***********************************************************************
        //【シリアルポートの初期設定処理】
        //***********************************************************************
        public Boolean InitializeSerialPort(int iCom)
        {
            COMPORT = PortName[iCom];
            Sensor.PortName = COMPORT;
            Sensor.ReadBufferSize = RxBufSize;              //
            Sensor.WriteBufferSize = TxBufSize;             //
            Sensor.BaudRate = 115200;                       //【ボーレート115200bps】
            Sensor.DataBits = 8;                            //【データ8bits】
            Sensor.StopBits = System.IO.Ports.StopBits.One; //【ストップビット1bit】
            Sensor.Parity = System.IO.Ports.Parity.None;    //【パリティーチェックなし】
            Sensor.ReadTimeout = 50;                        //【読出しは50ミリ秒でタイムアウト】
            Sensor.WriteTimeout = 10;                       //【書込みは10ミリ秒でタイムアウト】
            Sensor.Handshake = Handshake.None;
            try { Sensor.Open(); }
            catch (Exception) { return false; }
            return true;
        }

        //***********************************************************************
        //【シリアルポートを解放する】
        //***********************************************************************
        public void TerminateSerialPort()
        {
            if (Sensor.IsOpen) 
            {
                SendCommand(CMD_Stop);
                while (Sensor.BytesToRead > 0)
                {
                    System.Threading.Thread.Sleep(250); //【0.250秒Sleep】
                    Sensor.DiscardOutBuffer();
                    Sensor.DiscardInBuffer();
                    System.Threading.Thread.Sleep(250); //【0.250秒Sleep】
                }
                Sensor.Close();
            }
        }

        //***********************************************************************
        //【波形表示処理】
        //***********************************************************************
        public void DrawWaves(Graphics g)
        {
            g.Clear(Color.PeachPuff);
            Disp[0].DisplayGrid(g, Color.LightPink);
            if (Chk[0]) { Disp[0].DrawSignalLines(g, Color.Red, Zoom[0]); }
            if (Chk[1]) { Disp[1].DrawSignalLines(g, Color.Yellow, Zoom[1]); }
            if (Chk[2]) { Disp[2].DrawSignalLines(g, Color.Green, Zoom[2]); }
            if (Chk[3]) { Disp[3].DrawSignalLines(g, Color.Blue, Zoom[3]); }
        }

        //*************************************************************
        //***【測定開始】測定時間を秒単位で指定して測定開始
        //*************************************************************
        public void StartSampling(int n)
        {
            ClearSampleData(n * nSPS);
            nSampleCount = 0;
            Flag = true;
        }

        //***********************************************************************
        //【非接触バイタルセンサのA/D変換データの受信イベント処理】
        //***********************************************************************
        private void Sensor_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            int D;
            while (Sensor.BytesToRead >= nBlock) //【0.2秒間隔でイベントが発生する様に設定】
            {
                for (int i = 0; i < nData; i++ )
                {
                    Sensor.Read(RxBuf, 0, nBytes); //【12バイト取得】
                    //【A/D変換値をバイナリで取得】.Value: binary,  .Voltage: 電圧値(0.0～3.3V)
                    D = (int)RxBuf[0] + (int)RxBuf[1] * 256; if (fRev[0]) { D = Range - D; }
                    CH[0].Load(D);
                    D = (int)RxBuf[2] + (int)RxBuf[3] * 256; if (fRev[1]) { D = Range - D; }
                    CH[1].Load(D);
                 // D = (int)RxBuf[4] + (int)RxBuf[5] * 256; if (fRev[2]) { D = Range - D; }
                 // CH[2].Load(D);
                 // D = (int)RxBuf[6] + (int)RxBuf[7] * 256; if (fRev[3]) { D = Range - D; }
                 // CH[3].Load(D);
                 // D = (int)RxBuf[8] + (int)RxBuf[9] * 256; if (fRev[4]) { D = Range - D; }
                 // CH[4].Load(D);
                 // D = (int)RxBuf[10] + (int)RxBuf[11] * 256; if (fRev[5]) { D = Range - D; }
                 // CH[5].Load(D);

                    Disp[0].Load(CH[0].Value);   //【Ch0==>表示用FIFO】
                    Disp[1].Load(CH[1].Value);   //【Ch1==>表示用FIFO】
                 // Est.Estimation(CH[0].Value); //
                 // Disp[2].Load(Est.C);         //
                 // HBR.Load(Est.C);             //
                  mEst.Load(CH[0].Value);      //【最小２乗法で近似、加速度を推定】
                  Disp[2].Load(mEst.P1);       //【Ch0.加速度==>表示用FIFO】
                  HBR.Load(mEst.P1);           //【QRSフィルタで心拍数を求める】
                    Disp[3].Load(HBR.QRS);       //【矩形波相関==>表示用FIFO】

                    if (Flag) 
                    {
                        Sample0[nSampleCount] = CH[0].Value;       //【心電図】
                        Sample1[nSampleCount] = mEst.P1;           //【最小二乗法によるノイズ除去後の波形】
                        Sample2[nSampleCount] = HBR.QRS;           //【矩形波相関フィルタ】
                        Sample3[nSampleCount] = HBR.HeartBeatRate; //【瞬時心拍数】
                        Sample4[nSampleCount] = CH[1].Value;       //【赤外線脈波】
                        nSampleCount++;
                        if(nSampleCount==nSample)
                        {
                            SendCommand(CMD_Stop);
                            CreateDataFile();
                            TerminateSerialPort();
                            nSampleCount = 0;
                            Flag = false;
                            return;
                        }
                    }
                }
                ReceivedBytes = Sensor.BytesToRead;
            }
        }

        //***********************************************************************
        //【A/Dコンバータへのコマンド送信処理】
        //***********************************************************************
        private void SendCommand(byte cmd)
        {
            try
            {
                TxBuf[0] = cmd;
                Sensor.Write(TxBuf, 0, 1);
            }
            catch { }
        }

        //*************************************************************
        //***【測定データの初期化処理】
        //*************************************************************
        private void ClearSampleData(int n)
        {
            nSample = n;
            Sample0 = new double[nSample];
            Sample1 = new double[nSample];
            Sample2 = new double[nSample];
            Sample3 = new double[nSample];
            Sample4 = new double[nSample];
            for (int i = 0; i < nSample; i++)
            {
                Sample0[i] = 0.0;
                Sample1[i] = 0.0;
                Sample2[i] = 0.0;
                Sample3[i] = 0.0;
                Sample4[i] = 0.0;
            }
        }

        //*************************************************************
        //***【.csvファイル作成】
        //*************************************************************
        private void CreateDataFile()
        {
            //【現在の日付と時刻を取得してファイル名を作る】
            DateTime dtNow = DateTime.Now;
            String filename = @"../../VitalData"
                            + dtNow.Year.ToString("0000")
                            + dtNow.Month.ToString("00")
                            + dtNow.Day.ToString("00") + "_"
                            + dtNow.Hour.ToString("00")
                            + dtNow.Minute.ToString("00")
                            + dtNow.Second.ToString("00")
                            + ".csv";
            //【】
            System.Text.StringBuilder sBuf = new StringBuilder();
            sBuf.Length = 0;
            sBuf.Append(title);
            for (int i = 0; i < nSample; i++)
            {
                sBuf.Append(i.ToString() + ", ");
                sBuf.Append((i * 0.00125).ToString() + ", ");
                sBuf.Append(Sample0[i].ToString() + ", ");    //【ECG生波形】
                sBuf.Append(Sample1[i].ToString() + ", ");    //【基底遷移ノイズ除去】
                sBuf.Append(Sample2[i].ToString() + ", ");    //【矩形波相関フィルタ】
                sBuf.Append(Sample3[i].ToString() + ", ");    //【心拍数】
                sBuf.Append(Sample4[i].ToString() + "\r\n");  //【赤外線脈波】
            }
            System.IO.StreamWriter sw = new System.IO.StreamWriter(
                filename,
                false,
                System.Text.Encoding.GetEncoding("shift_jis"));
            sw.Write(sBuf.ToString());
            sw.Close();  //閉じる
            sBuf.Clear();
        }
    }
}
