﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing; //【Point】
using System.Drawing.Imaging;

namespace NTL.KinectLib
{
    //***************************************************************
    //【波形表示処理】入力：double型
    //***************************************************************
    public class DisplayWave
    {
        public const int nSPS = 800;
        public const int nDCoffset = (int)(0.5 * nSPS); //【DCオフセット演算サンプル数】
        public const int nWave = 5 * nSPS;     //【波形表示サンプル数】
        public const float K1 = 1.0f / (float)nDCoffset; //【変換係数】
        public const int Width = nWave;
        public const int Height = 480;

        public int pWave0;      //【波形表示用FIFOポインタ】初期値：nDCoffset
        public int pWave1;      //【波形表示用FIFOポインタ】初期値：0
        public double Sum;      //【移動累積加算値】
        public double DCOffset; //【FIFOのDCオフセット電圧】
        public double[] Wave;   //【波形表示用FIFO】
        public Point[] Points;  //【波形表示用Point配列】
        public Bitmap Bmp;      //【波形表示Bitmap】

        public void Initialize()
        {
            Wave = new double[nWave];  //【波形表示用FIFO】
            Points = new Point[nWave]; //【波形表示用Point配列】
            Bmp = new Bitmap(Width, Height, PixelFormat.Format32bppRgb); //【波形表示Bitmap】
            for (int i = 0; i < nWave; i++) { Wave[i] = 0.0; Points[i].X = i; Points[i].Y = 0; }
            pWave0 = nDCoffset;
            pWave1 = 0;
            Sum = 0.0;
            DCOffset = 0.0;
        }

        public void Load(double DataIn)
        {
            Sum += DataIn - Wave[pWave1];
            Wave[pWave0] = DataIn;
            DCOffset = Sum * K1;
            pWave0++; if (pWave0 == nWave) { pWave0 = 0; }
            pWave1++; if (pWave1 == nWave) { pWave1 = 0; }
        }

        public void DrawSignalLines(Graphics g, Color lc, int zoom)
        {
            if ((zoom < 0) || (zoom > 7)) { return; }

            const float PenWidth = 5.0f;
            const double CenterLine = Height / 2.0;
            double[] Z = new double[] { 50000.0, 25000.0, 12500.0, 5000.0, 2500.0, 1250.0, 500.0 };
            int i, j;
            double Ofs = DCOffset;
            double Dy = CenterLine / Z[zoom];
            for (i = 0, j = pWave0; i < nWave; i++)
            {
                Points[i].Y = (int)(-(Wave[j] - Ofs) * Dy + CenterLine); //【中央値をOffsetLevelとする】
                j++; if (j == nWave) { j = 0; } //【FIFOポインタを更新】
            }
            g.DrawLines(new Pen(lc, PenWidth), Points);
        }

        public void DisplayGrid(Graphics g, Color lc)
        {
            const float PenWidth = 3.0f;
            for (int i = 0; i < nWave - 1; i += (int)(nWave * 0.1)) { g.DrawLine(new Pen(lc, PenWidth), i, 0, i, (int)Height); }
            g.DrawLine(new Pen(lc, PenWidth), nWave - 1, 0, nWave - 1, (int)Height);
            for (int i = 0; i < (int)Height; i += (int)(Height * 0.1)) { g.DrawLine(new Pen(lc, PenWidth), 0, i, nWave - 1, i); }
        }
    }
}
