﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NTL.KinectLib
{
    //***************************************************************
    //【最小２乗法による加速度推定】O(t) = P3 * t^2 + P2 * t + P1    行列3x3, 入力：double型
    //***************************************************************
    public class EstimateParameter
    {
        public int nFIFO; // 30;
        public int nSPS;
        public double P1, P2, P3; //【未知数】P1:現在値、P2:速度、P3:加速度
        private double[] FIFO;
        private int pW;
        private int pR;
        private double[] S = new double[5];
        private double A11, A12, A13, A21, A22, A23, A31, A32, A33; //【最小二乗法行列】
        private double K; //【行列式の逆数】 1.0/detA
        private double K2, K3;
        private double M11, M12, M13, M21, M22, M23, M31, M32, M33; //【逆行列】
     // private double T11, T12, T13, T21, T22, T23, T31, T32, T33; //【逆行列テスト用】
        private double Y0, Y1, Y2;

        //***************************************************************
        //【初期化処理】最小２乗法による逆行列Mを演算
        //***************************************************************
        public void Initialize(int n, int m)
        {
            nSPS = m;
            K2 = (double)nSPS;
            K3 = 2.0 * (double)nSPS * (double)nSPS;
            nFIFO = n;
            FIFO = new double[nFIFO];
            long j = 0;
            for (int i = 0; i < S.Length; i++) { S[i] = 0.0; }
            for (int i = 0; i < FIFO.Length; i++)
            {
                FIFO[i] = 0.0;
                S[0] += 1.0;
                S[1] += (double)j;
                S[2] += (double)j * (double)j;
                S[3] += (double)j * (double)j * (double)j;
                S[4] += (double)j * (double)j * (double)j * (double)j;
                j--;
            }
            A11 = S[0]; A12 = S[1]; A13 = S[2];
            A21 = S[1]; A22 = S[2]; A23 = S[3];
            A31 = S[2]; A32 = S[3]; A33 = S[4];
            K = 1.0 / (A11 * A22 * A33 + A21 * A32 * A13 + A31 * A12 * A23 - A11 * A32 * A23 - A31 * A22 * A13 - A21 * A12 * A33); //【行列式の逆数】
            M11 = (A22 * A33 - A23 * A32) * K; M12 = (A13 * A32 - A12 * A33) * K; M13 = (A12 * A23 - A13 * A22) * K;
            M21 = (A23 * A31 - A21 * A33) * K; M22 = (A11 * A33 - A13 * A31) * K; M23 = (A13 * A21 - A11 * A23) * K;
            M31 = (A21 * A32 - A22 * A31) * K; M32 = (A12 * A31 - A11 * A32) * K; M33 = (A11 * A22 - A12 * A21) * K;
            pW = 0;

         // T11 = A11 * M11 + A12 * M21 + A13 * M31; // 1
         // T12 = A11 * M12 + A12 * M22 + A13 * M32; // 0
         // T13 = A11 * M13 + A12 * M23 + A13 * M33; // 0
         // T21 = A21 * M11 + A22 * M21 + A23 * M31; // 0
         // T22 = A21 * M12 + A22 * M22 + A23 * M32; // 1
         // T23 = A21 * M13 + A22 * M23 + A23 * M33; // 0
         // T31 = A31 * M11 + A32 * M21 + A33 * M31; // 0
         // T32 = A31 * M12 + A32 * M22 + A33 * M32; // 0
         // T33 = A31 * M13 + A32 * M23 + A33 * M33; // 1
        }

        //***************************************************************
        //【逆行列Mを用いて最小２乗法で未知数を演算】O(t) = P3 * t^2 + P2 * t + P1
        //***************************************************************
        public void Load(double DataIn)
        {
            pR = pW;
            FIFO[pW] = DataIn;
            pW++; if (pW == nFIFO) { pW = 0; }

            int j = 0;
            double D;
            Y0 = 0.0; Y1 = 0.0; Y2 = 0.0;
            for (int i = 0; i < nFIFO; i++)
            {
                D = FIFO[pR];
                Y0 += D;
                Y1 += (double)j * D;
                Y2 += (double)j * (double)j * D;
                j--;
                pR--; if (pR < 0) { pR = nFIFO - 1; }
            }
            P1 = M11 * Y0 + M12 * Y1 + M13 * Y2;
            P2 = (M21 * Y0 + M22 * Y1 + M23 * Y2) * K2;
            P3 = (M31 * Y0 + M32 * Y1 + M33 * Y2) * K3;
        }
    }
}
