﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NTL.KinectLib
{
    //***************************************************************
    //【矩形波相関トリガ処理】入力：double型
    //***************************************************************
    public class QRSFilter
    {
        private const int nFIFO = 64;
        private const int nQRS4 = nFIFO / 4;
        private const double K1 = 1.0 / (double)nFIFO;
        private const long Period = 800 * 5;
        private int p0, p1, p2;
        private double[] FIFO = new double[nFIFO];
        private double Sum;
        private double maxQRS;
        public double QRS;

        private const double Vmax = 100000.0;
        private const double Delta = 0.00125;
        private double pPeekTime;
        private double cPeekTime;
        private long mCount;
        private long cCount;
        private double Lmax;
        private double LThreshold;
        private int Mode;
        public double Time;
        public double Threshold;
        public double HeartBeatRate;

        public double aveHeartBeatRate;

        public void Initialize()
        {
            ClearFIFO(); //【矩形波相関フィルタの初期化処理】
            Threshold = 0.5 * Vmax; //【最初のしきい値を範囲の1/2に設定】
            LThreshold = Threshold;
            Mode = 0;

            mCount = 0;                //【測定カウント】
            cCount = mCount + Period;
            Lmax = -Vmax;
            maxQRS = -Vmax;
            pPeekTime = 0.0; cPeekTime = 0.0;
            HeartBeatRate = 0.0;
            aveHeartBeatRate = 0.0;
        }

        private void ClearFIFO()
        {
            p0 = 0;
            p1 = nQRS4 * 3;
            p2 = nQRS4 * 1;
            for (int i = 0; i < FIFO.Length; i++) { FIFO[i] = 0.0; }
            Sum = 0.0;
            QRS = 0.0;
        }

        public void Load(double DataIn)
        {
            //【矩形波相関フィルタの演算】
            Sum += -DataIn + 2.0 * FIFO[p1] - 2.0 * FIFO[p2] + FIFO[p0];
            FIFO[p0] = DataIn;
            QRS = Sum * K1;
            p0++; if (p0 == nFIFO) { p0 = 0; }
            p1++; if (p1 == nFIFO) { p1 = 0; }
            p2++; if (p2 == nFIFO) { p2 = 0; }

             //【心拍数の検出】
            Time = (double)mCount * Delta;
            if (QRS > Threshold)
            {
                if (QRS >= maxQRS)
                {
                    maxQRS = QRS;
                }
                else
                {
                    if (Mode == 0) 
                    {
                        pPeekTime = cPeekTime; //【前回のQRS群の時刻を保存】
                        cPeekTime = Time;      //【今回のQRS群の時刻を計算】
                        if (pPeekTime > 1.0)
                        {
                            HeartBeatRate = 60.0 / (cPeekTime - pPeekTime); //【瞬時心拍数を演算】
                        }
                        Mode = 1;
                    }
                }
            }
            else
            {
                if (QRS < 0.0) { Threshold = LThreshold; }
                maxQRS = -Vmax;
                Mode = 0;
            }
            if (QRS > Lmax) { Lmax = QRS; }
            if (mCount == cCount) 
            {
                LThreshold = Lmax * 0.7;
                cCount = mCount + Period;
                Lmax = -Vmax;
            }
            mCount++;
        }
    }
}
