﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices; //【exportDLL】

namespace VitalMonitor
{
    class Win32API
    {
        /*
        //***********************************************************************
        //【Win32API】
        //***********************************************************************
        [DllImport("kernel32.dll")]
        private extern static bool Beep(uint dwFreq, uint dwDuration);

        [DllImport("kernel32.dll")]
        public extern static uint CreateFile(string CommName, uint DesiredAccess, uint ShareMode, uint SecurityAttributes, uint CreationDisposition, uint FlagsAndAttributes, uint TemplateFile);
        [DllImport("kernel32.dll")]
        public extern static uint CloseHandle(uint hCom);

        public const uint GENERIC_READ = 0x80000000;
        public const uint GENERIC_WRITE = 0x40000000;
        public const uint GENERIC_EXECUTE = 0x20000000;
        public const uint GENERIC_ALL = 0x10000000;

        public const uint CREATE_NEW = 1;
        public const uint CREATE_ALWAYS = 2;
        public const uint OPEN_EXISTING = 3;
        public const uint OPEN_ALWAYS = 4;
        public const uint TRUNCATE_EXISTING = 5;

        public const uint FILE_ATTRIBUTE_READONLY = 0x1;
        public const uint FILE_ATTRIBUTE_HIDDEN = 0x2;
        public const uint FILE_ATTRIBUTE_SYSTEM = 0x4;
        public const uint FILE_ATTRIBUTE_DIRECTORY = 0x10;
        public const uint FILE_ATTRIBUTE_ARCHIVE = 0x20;
        public const uint FILE_ATTRIBUTE_NORMAL = 0x80;
        public const uint FILE_ATTRIBUTE_TEMPORARY = 0x100;
        public const uint FILE_ATTRIBUTE_COMPRESSED = 0x800;

        //***********************************************************************
        //【SetupComm】 通信ポートの受信バッファ、及び送信バッファを確保する。
        //***********************************************************************
        [DllImport("kernel32.dll")]
        public extern static uint SetupComm(uint hCom, uint InQueue, uint OutQueue);

        //***********************************************************************
        //【WriteFile】 通信ポートの送信バッファに対する書き込み。
        //【ReadFile】  通信ポートの受信バッファからの読み出し
        //***********************************************************************
        [DllImport("kernel32.dll")]
        public extern static uint WriteFile(uint hCom, byte[] Buffer, uint NumberOfBytesToWrite, out uint NumberOfBytesWritten, uint lpOverlapped);
        [DllImport("kernel32.dll")]
        public extern static uint ReadFile(uint hCom, out byte[] Buffer, uint NumberOfBytesToRead, out uint NumberOfBytesRead, uint lpOverlapped);

        //***********************************************************************
        //【GetCommState】 通信ポートのカレント状態をDCB構造体に取得する。
        //【SetCommState】 通信ポートのカレント状態を指定したDCB構造体に従って設定する。
        //***********************************************************************
        public extern static uint GetCommState(uint hCom, out DCBstructure DCB);
        public extern static uint SetCommState(uint hCom, DCBstructure DCB);

        //***********************************************************************
        //【BuildCommDCB】           通信ポートにおけるDCB構造体の構築設定。
        //【BuildCommDCBAndTimeouts】通信ポートにおけるDCB構造体とCOMMTIMEOUTS構造体の構築設定。
        //***********************************************************************
        public extern static uint BuildCommDCB(string Def, DCBstructure DCB);
        public extern static uint BuildCommDCBAndTimeouts(string Def, DCBstructure DCB, COMMTIMEOUTSstructure CommTimeouts);
        public struct DCBstructure // DCB構造体の定義は以下の通り。
        {
            uint DCBlength; //   sizeof(DCB)
            uint BaudRate;  // ◎current baud rate
            uint Flags;     // ◎FlagsのBit Assignは以下の通り。0x0000011を推奨。
            // fBinary           :1   '   binary mode, no EOF check
            // fParity           :1   '   enable parity checking
            // fOutxCtsFlow      :1   '   CTS output flow control
            // fOutxDsrFlow      :1   '   DSR output flow control
            // fDtrControl       :2   '   DTR flow control type
            // fDsrSensitivity   :1   '   DSR sensitivity
            // fTXContinueOnXoff :1   '   XOFF continues Tx
            // fOutX             :1   '   XON/XOFF out flow control
            // fInX              :1   '   XON/XOFF in flow control
            // fErrorChar        :1   '   enable error replacement
            // fNull             :1   '   enable null stripping
            // fRtsControl       :2   '   RTS flow control
            // fAbortOnError     :1   '   abort reads/writes on error
            // fDummy2           :17  '   reserved
            ushort wReserved1; //   not currently used
            ushort XonLim;     //   transmit XON threshold
            ushort XoffLim;    //   transmit XOFF threshold
            byte ByteSize;     // ◎number of bits/byte, 4-8
            byte Parity;       // ◎0-4=no,odd,even,mark,space
            byte StopBits;     // ◎0,1,2 = 1, 1.5, 2
            byte XonChar;      //   Tx and Rx XON character
            byte XoffChar;     //   Tx and Rx XOFF character
            byte ErrorChar;    //   error replacement character
            byte EofChar;      //   end of input character
            byte EvtChar;      //   received event character
            ushort Reserved2;  //   reserved; do not use
        }
        // Comm Baud Rate indices　   [BaudRateに設定可能なボーレート]
        public const uint CBR_110 = 110; //       ' ○
        public const uint CBR_300 = 300; //       ' ○
        public const uint CBR_600 = 600; //       ' ○
        public const uint CBR_1200 = 1200; //     ' ○
        public const uint CBR_2400 = 2400; //     ' ○
        public const uint CBR_4800 = 4800; //     ' ○
        public const uint CBR_9600 = 9600; //     ' ○
        public const uint CBR_14400 = 14400; //   ' ○
        public const uint CBR_19200 = 19200; //   ' ○
        public const uint CBR_38400 = 38400; //   ' ○
        public const uint CBR_56000 = 56000; //   ' ○
        public const uint CBR_57600 = 57600; //   ' ○
        public const uint CBR_115200 = 115200; // ' ○(NEC PC9821ｼﾘｰｽﾞの最大値)
        public const uint CBR_128000 = 128000; // ' ×(予約)
        public const uint CBR_256000 = 256000; // ' ×(予約)
        // DCB構造体のFlagsのfDtrControlに設定可能な数値
        public const uint DTR_CONTROL_DISABLE = 0x0;
        public const uint DTR_CONTROL_ENABLE = 0x1;
        public const uint DTR_CONTROL_HANDSHAKE = 0x2;
        // DCB構造体のFlagsのfRtsControlに設定可能な数値
        public const uint RTS_CONTROL_DISABLE = 0x0;
        public const uint RTS_CONTROL_ENABLE = 0x1;
        public const uint RTS_CONTROL_HANDSHAKE = 0x2;
        public const uint RTS_CONTROL_TOGGLE = 0x3;

        //***********************************************************************
        //【GetCommTimeouts】 通信ポートにおけるCOMMTIMEOUTS構造体の取得。
        //【SetCommTimeouts】 通信ポートにおけるCOMMTIMEOUTS構造体の設定。
        //***********************************************************************
        public extern static uint GetCommTimeouts(uint hCom, out COMMTIMEOUTSstructure CommTimeouts);
        public extern static uint SetCommTimeouts(uint hCom, COMMTIMEOUTSstructure lpCommTimeouts);
        public struct COMMTIMEOUTSstructure    // COMMTIMEOUTS構造体の定義は以下の通り。
        {
            uint ReadIntervalTimeout;         // ReadFile中での受信文字間の許容時間   (mSec) 未使用時は0
            uint ReadTotalTimeoutMultiplier;  // ReadFile中での受信文字数×Multiplier (mSec) 未使用時は0
            uint ReadTotalTimeoutConstant;    // ReadFile中での受信許容時間           (mSec) 未使用時は0
            uint WriteTotalTimeoutMultiplier; // WriteFile中での送信文字数×Multiplier(mSec) 未使用時は0
            uint WriteTotalTimeoutConstant;   // WriteFile中での送信許容時間          (mSec) 未使用時は0
        }

        //***********************************************************************
        //【ClearCommError】 通信ポートのERRORのクリア、及び回線、通信バッファの状態取得。
        //***********************************************************************
        public extern static uint ClearCommError(uint hCom, out uint Errors, COMSTATstructure Stat);
        // Error Flags
        public const uint CE_RXOVER = 0x1;   // 受信キューにオーバーフロー発生
        public const uint CE_OVERRUN = 0x2;  // 受信オーバーラン発生（ハードウェア処理速度を超えて入力し、バッファに格納失敗）
        public const uint CE_RXPARITY = 0x4; // 受信パリティーチェックエラー発生
        public const uint CE_FRAME = 0x8;    // 受信フレーミングエラー発生（）
        public const uint CE_BREAK = 0x10;   //  Break Detected
        public const uint CE_TXFULL = 0x100; //  TX Queue is full
        public const uint CE_PTO = 0x200;    //  LPTx Timeout
        public const uint CE_IOE = 0x400;    //  LPTx I/O Error
        public const uint CE_DNS = 0x800;    //  LPTx Device not selected
        public const uint CE_OOP = 0x1000;   //  LPTx Out-Of-Paper
        public const uint CE_MODE = 0x8000;  //  Requested mode unsupported
        //
        public struct COMSTATstructure   // COMSTAT構造体の定義を以下の通り。
        {
            uint Flags;
            // fCtsHold   :1  ' Tx waiting for CTS signal
            // fDsrHold   :1  ' Tx waiting for DSR signal
            // fRlsdHold  :1  ' Tx waiting for RLSD signal
            // fXoffHold  :1  ' Tx waiting, XOFF char rec'd
            // fXoffSent  :1  ' Tx waiting, XOFF char sent
            // fEof       :1  ' EOF character sent
            // fTxim      :1  ' character waiting for Tx
            // fReserved  :25 ' reserved
            uint InQue;
            uint OutQue;
        }

        //***********************************************************************
        //【PurgeComm】 通信ポートにおけるPURGE設定。
        //***********************************************************************
        public extern static uint PurgeComm(uint hCom, uint Flags);
        public const uint PURGE_TXABORT = 0x1; // 実施中の送信処理のアボート
        public const uint PURGE_RXABORT = 0x2; // 実施中の受信処理のアボート
        public const uint PURGE_TXCLEAR = 0x4; // 送信バッファのクリア(OutBufferCount <= 0)
        public const uint PURGE_RXCLEAR = 0x8; // 受信バッファのクリア (InBufferCount <= 0)
        */
    }
}
